#!/usr/bin/perl

# searches.pl
# by Jon C. Jensen, jon@viscom.byu.edu
# v1.0 - 1999 February
# v1.1 - 1999 March 29 - added minimum hit limit capability, sorting options,
#                        CGI parameter passing, combined shell & CGI versions

# path of system logfile names to glob (required)
$logpattern = "/var/log/messages*";


# running through CGI?
if ($ENV{'GATEWAY_INTERFACE'}) {
 $cgi = 1;
 # read in CGI options
 if ($ENV{'REQUEST_METHOD'} eq "GET") {
  $in = $ENV{'QUERY_STRING'};
 } elsif ($ENV{'REQUEST_METHOD'} eq "POST") {
  read(STDIN,$in,$ENV{'CONTENT_LENGTH'});
 }
 # clean up encoding & parse options
 if ($in) {
  $in =~ s/\+/ /g;
  @optionlist = split(/&/, $in);
  foreach (@optionlist) {
   ($key, $value) = split(/=/, $_, 2);
   $key   = cgidecode($key);
   $value = cgidecode($value);
   $options{$key} = $value;
  }
 }
}


##### configuration (someday allow for command-line options as well):

# htdig configuration(s) to report for, separated by spaces
# (leave empty to report for all at once)
$forconf = split(/ /,$options{"forconf"});

# minimum number of searches per word to show in report 
$minwordhits = $options{"minwordhits"};

# sort words alphabetically by search word? (otherwise by frequency)
$wordsortalpha = $options{"wordsortalpha"};

# minimum number of hits per host to show in report
$minhosthits = $options{"minhosthits"};

# sort hosts alphabetically by hostname? (otherwise by frequency)
$hostsortalpha = $options{"hostsortalpha"};

# sort search types alphabetically? (otherwise by frequency)
$typesortalpha = $options{"typesortalpha"};

##### end config section


# process each logfile in turn

foreach $logfile (glob($logpattern)) {

 open IN, $logfile || warn "Couldn't open $logfile: $!";

 # check each line of logfile for htsearch process messages
 while (<IN>) {

  # skip log entries that don't look similar to this:
  # Feb 22 18:08:08 phantom htsearch[3703]: 128.187.52.123 [htdig] (and) [golf camps] [(golf or golfing) and (camps or camp)] (11/10) - 1 -- http://vc.clubs.byu.edu/htdig/
  next unless /\s+htsearch\[\d+\]:\s+([^\s]+)\s+\[([^\]]+)\]\s+\(([^)]+)\)\s+\[([^\]]+)\]/;
  $hostname    = $1;
  $htdigconf   = $2;
  $searchtype  = $3;
  $searchwords = $4;

  # skip entry if not for desired htdig search configuration
  if ($forconf) {next unless $forconf eq $htdigconf;}

  # increase total searches counter
  ++$totalsearches;

  # increase counter for this host
  ++$hostfreq{$hostname};

  # increase counter for this search type (and, or, boolean)
  ++$typefreq{$searchtype};

  # split search string into words and increase counter for each 
  foreach $word (split(/ /,$searchwords)) {++$wordfreq{$word}}
 }
 close IN || die;
}

# initialize some HTML variables if CGI, otherwise they'll be blank
if ($cgi) {
 $h2_on = "<h2>"; $h2_off = "</h2>";
 $p_on = "<p>"; $p_off = "</p>";
 $br = "<br>";
}

# output report

print "Content-type: text/html\n\n" if $cgi;

$title = "ht://Dig search statistics";
if ($cgi) {
 print "<html>\n<head>\n<title>$title</title>\n</head>\n<body>\n\n";
 print "<h1>$title</h1>\n\n";
} else {
 print "\n$title\n\n";
}

if ($totalsearches > 0) {

 print "${h2_on}Total number of searches: $totalsearches${h2_off}\n\n";

 print "${h2_on}Search type frequencies:$h2_off\n";
 print "<p>\n" if $cgi;
 $first = 1;

 if ($typesortalpha) {@results = sort keys %typefreq} else {@results = valuesort(\%typefreq)} 
 foreach $type (@results) {
  if ($first) {$first = 0} else {print "$br\n";}
  print "$type ($typefreq{$type})";
 }
 print "$p_off\n\n";
 
 print "${h2_on}Search word frequencies:$h2_off\n";
 print "<p>\n" if $cgi;
 $first = 1;
 if ($wordsortalpha) {@results = sort keys %wordfreq} else {@results = valuesort(\%wordfreq)}
 foreach $word (@results) {
  next unless $wordfreq{$word} >= $minwordhits;
  if ($first) {$first = 0} else {print "$br\n";}
  print "$word ($wordfreq{$word})";
 }
 print "$p_off\n\n";
 
 print "${h2_on}Search host frequencies:$h2_off\n";
 print "<p>\n" if $cgi;
 $first = 1;
 if ($hostsortalpha) {@results = sort keys %hostfreq} else {@results = valuesort(\%hostfreq)}
 foreach $host (@results) {
  next unless $hostfreq{$host} >= $minhosthits;
  if ($first) {$first = 0} else {print "$br\n";}
  print "$host ($hostfreq{$host})";
 }
 print "$p_off\n\n";
 
 print "</body></html>\n" if $cgi;

} else {

 print "${h2_on}No searches were made.$h2_off\n";
}


# subroutine to sort hash keys by their respective associated values
# (adapted from Programming Perl by Larry Wall)

sub valuesort {
 my $hashref = shift;
 return sort { $$hashref{$b} <=> $$hashref{$a} } keys %$hashref;
}


# subroutine to decode the "%2E" etc. hex encoding passed in through CGI

sub cgidecode {
 local $_ = shift;
 s/%([a-fA-F0-9][a-fA-F0-9])/pack("c",hex($1))/ge;
 return $_;
}

